@@ -128,6 +128,20 @@ static void average_copy(const snd_pcm_channel_area_t *dst_areas,
 	}
 }
 
+static void upmix_1_to_71(snd_pcm_upmix_t *mix ATTRIBUTE_UNUSED,
+			  const snd_pcm_channel_area_t *dst_areas,
+			  snd_pcm_uframes_t dst_offset,
+			  const snd_pcm_channel_area_t *src_areas,
+			  snd_pcm_uframes_t src_offset,
+			  snd_pcm_uframes_t size)
+{
+	int i;
+	for (i = 0; i < 8; i++)
+		snd_pcm_area_copy(dst_areas + i, dst_offset,
+				  src_areas, src_offset,
+				  size, SND_PCM_FORMAT_S16);
+}
+
 static void upmix_1_to_51(snd_pcm_upmix_t *mix ATTRIBUTE_UNUSED,
 			  const snd_pcm_channel_area_t *dst_areas,
 			  snd_pcm_uframes_t dst_offset,
@@ -156,6 +170,24 @@ static void upmix_1_to_40(snd_pcm_upmix_t *mix ATTRIBUTE_UNUSED,
 				  size, SND_PCM_FORMAT_S16);
 }
 
+static void upmix_2_to_71(snd_pcm_upmix_t *mix,
+			  const snd_pcm_channel_area_t *dst_areas,
+			  snd_pcm_uframes_t dst_offset,
+			  const snd_pcm_channel_area_t *src_areas,
+			  snd_pcm_uframes_t src_offset,
+			  snd_pcm_uframes_t size)
+{
+	snd_pcm_areas_copy(dst_areas, dst_offset, src_areas, src_offset,
+			   2, size, SND_PCM_FORMAT_S16);
+	delayed_copy(mix, dst_areas + 2, dst_offset, src_areas, src_offset,
+		     size);
+	average_copy(dst_areas + 4, dst_offset, src_areas, src_offset,
+		     2, size);
+	snd_pcm_areas_copy(dst_areas + 6, dst_offset, src_areas, src_offset,
+			   2, size, SND_PCM_FORMAT_S16);
+	
+}
+
 static void upmix_2_to_51(snd_pcm_upmix_t *mix,
 			  const snd_pcm_channel_area_t *dst_areas,
 			  snd_pcm_uframes_t dst_offset,
@@ -260,13 +292,26 @@ static void upmix_6_to_51(snd_pcm_upmix_t *mix ATTRIBUTE_UNUSED,
 			   6, size, SND_PCM_FORMAT_S16);
 }
 
-static const upmixer_t do_upmix[6][2] = {
-	{ upmix_1_to_40, upmix_1_to_51 },
-	{ upmix_2_to_40, upmix_2_to_51 },
-	{ upmix_3_to_40, upmix_3_to_51 },
-	{ upmix_4_to_40, upmix_4_to_51 },
-	{ upmix_4_to_40, upmix_5_to_51 },
-	{ upmix_4_to_40, upmix_6_to_51 },
+static void upmix_8_to_71(snd_pcm_upmix_t *mix ATTRIBUTE_UNUSED,
+			  const snd_pcm_channel_area_t *dst_areas,
+			  snd_pcm_uframes_t dst_offset,
+			  const snd_pcm_channel_area_t *src_areas,
+			  snd_pcm_uframes_t src_offset,
+			  snd_pcm_uframes_t size)
+{
+	snd_pcm_areas_copy(dst_areas, dst_offset, src_areas, src_offset,
+			   8, size, SND_PCM_FORMAT_S16);
+}
+
+static const upmixer_t do_upmix[8][3] = {
+	{ upmix_1_to_40, upmix_1_to_51, upmix_1_to_71 },
+	{ upmix_2_to_40, upmix_2_to_51, upmix_2_to_71 },
+	{ upmix_3_to_40, upmix_3_to_51, upmix_3_to_51 },
+	{ upmix_4_to_40, upmix_4_to_51, upmix_4_to_51 },
+	{ upmix_4_to_40, upmix_5_to_51, upmix_5_to_51 },
+	{ upmix_4_to_40, upmix_6_to_51, upmix_6_to_51 },
+	{ upmix_4_to_40, upmix_6_to_51, upmix_6_to_51 },
+	{ upmix_4_to_40, upmix_6_to_51, upmix_8_to_71 },
 };
 
 static snd_pcm_sframes_t
@@ -288,9 +333,18 @@ static int upmix_init(snd_pcm_extplug_t *ext)
 	snd_pcm_upmix_t *mix = (snd_pcm_upmix_t *)ext;
 	int ctype, stype;
 
-	stype = (ext->slave_channels == 6) ? 1 : 0;
+	switch (ext->slave_channels) {
+		case	6:
+			stype = 1;
+			break;
+		case 8:
+			stype = 2;
+			break;
+		default:
+			stype = 0;
+	}
 	ctype = ext->channels - 1;
-	if (ctype < 0 || ctype > 5) {
+	if (ctype < 0 || ctype > 7) {
 		SNDERR("Invalid channel numbers for upmix: %d", ctype + 1);
 		return -EINVAL;
 	}
@@ -317,10 +371,63 @@ static int upmix_close(snd_pcm_extplug_t *ext)
 	return 0;
 }
 
+#if SND_PCM_EXTPLUG_VERSION >= 0x10002
+static unsigned int chmap[8][8] = {
+	{ SND_CHMAP_MONO },
+	{ SND_CHMAP_FL, SND_CHMAP_FR },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR, SND_CHMAP_FC },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR, SND_CHMAP_FC, SND_CHMAP_LFE },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR, SND_CHMAP_FC, SND_CHMAP_LFE, SND_CHMAP_UNKNOWN },
+	{ SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR, SND_CHMAP_FC, SND_CHMAP_LFE, SND_CHMAP_SL, SND_CHMAP_SR },
+};
+
+static snd_pcm_chmap_query_t **upmix_query_chmaps(snd_pcm_extplug_t *ext ATTRIBUTE_UNUSED)
+{
+	snd_pcm_chmap_query_t **maps;
+	int i;
+
+	maps = calloc(9, sizeof(void *));
+	if (!maps)
+		return NULL;
+	for (i = 0; i < 8; i++) {
+		snd_pcm_chmap_query_t *p;
+		p = maps[i] = calloc(i + 1 + 2, sizeof(int));
+		if (!p) {
+			snd_pcm_free_chmaps(maps);
+			return NULL;
+		}
+		p->type = SND_CHMAP_TYPE_FIXED;
+		p->map.channels = i + 1;
+		memcpy(p->map.pos, &chmap[i][0], (i + 1) * sizeof(int));
+	}
+	return maps;
+}
+
+static snd_pcm_chmap_t *upmix_get_chmap(snd_pcm_extplug_t *ext)
+{
+	snd_pcm_chmap_t *map;
+
+	if (ext->channels < 1 || ext->channels > 8)
+		return NULL;
+	map = malloc((ext->channels + 1) * sizeof(int));
+	if (!map)
+		return NULL;
+	map->channels = ext->channels;
+	memcpy(map->pos, &chmap[ext->channels - 1][0], ext->channels * sizeof(int));
+	return map;
+}
+#endif /* SND_PCM_EXTPLUG_VERSION >= 0x10002 */
+
 static const snd_pcm_extplug_callback_t upmix_callback = {
 	.transfer = upmix_transfer,
 	.init = upmix_init,
 	.close = upmix_close,
+#if SND_PCM_EXTPLUG_VERSION >= 0x10002
+	.query_chmaps = upmix_query_chmaps,
+	.get_chmap = upmix_get_chmap,
+#endif
 };
 
 SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
@@ -328,7 +435,7 @@ SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
 	snd_config_iterator_t i, next;
 	snd_pcm_upmix_t *mix;
 	snd_config_t *sconf = NULL;
-	static const unsigned int chlist[2] = {4, 6};
+	static const unsigned int chlist[3] = {4, 6, 8};
 	unsigned int channels = 0;
 	int delay = 10;
 	int err;
@@ -352,6 +459,7 @@ SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
 				return err;
 			}
 			delay = val;
+			continue;
 		}
 		if (strcmp(id, "channels") == 0) {
 			long val;
@@ -361,8 +469,8 @@ SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
 				return err;
 			}
 			channels = val;
-			if (channels != 4 && channels != 6 && channels != 0) {
-				SNDERR("channels must be 4, 6 or 0");
+			if (channels != 4 && channels != 6 && channels != 0 && channels != 8) {
+				SNDERR("channels must be 4, 6, 8 or 0");
 				return -EINVAL;
 			}
 			continue;
@@ -398,7 +506,7 @@ SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
 
 	snd_pcm_extplug_set_param_minmax(&mix->ext,
 					 SND_PCM_EXTPLUG_HW_CHANNELS,
-					 1, 6);
+					 1, 8);
 	if (channels)
 		snd_pcm_extplug_set_slave_param_minmax(&mix->ext,
 						       SND_PCM_EXTPLUG_HW_CHANNELS,
@@ -406,7 +514,7 @@ SND_PCM_PLUGIN_DEFINE_FUNC(upmix)
 	else
 		snd_pcm_extplug_set_slave_param_list(&mix->ext,
 						     SND_PCM_EXTPLUG_HW_CHANNELS,
-						     2, chlist);
+						     3, chlist);
 	snd_pcm_extplug_set_param(&mix->ext, SND_PCM_EXTPLUG_HW_FORMAT,
 				  SND_PCM_FORMAT_S16);
 	snd_pcm_extplug_set_slave_param(&mix->ext, SND_PCM_EXTPLUG_HW_FORMAT,
